/********************************************************************************
 Author : CAC (China Applications Support Team) 

 Date :   May, 2014

 File name :  ADE7913.c 

 Description :	 ADE7913 SPI communication

 Hardware plateform : 	EVAL-ADuCM360MKZ and EVAL-ADE7913EBZ
********************************************************************************/

#include <ADuCM360.h>

#include "UrtLib.h"
#include "ClkLib.h"
#include "WdtLib.h"
#include "IntLib.h"
#include "DioLib.h"
#include "SpiLib.h"

#include "ADE7913.h"
#include "ADuCM360Driver.h"

/*******************************************************************
  @brief void  ADE7913WriteOperation(unsigned char Address, unsigned int SendValue)
         The write operation using ADE7913 SPI interface
  @param
         unsigned char Address:    register address
         unsigned int SendValue:   value to be written to register 
  @return   
         none
*******************************************************************/
void  ADE7913WriteOperation(unsigned char Address, unsigned int SendValue)
{
    unsigned int  SendCommand = 0;
    unsigned char SendTemp[2];
    unsigned char ReceiveTemp[2];  
    
    SendCommand = Address<<3;
    
    SendTemp[0] = SendCommand;
    SendTemp[1] = SendValue;
  
    DioClr(pADI_GP1,0x80);              //CS down
    SpiFunction(SendTemp, ReceiveTemp, 2);   
    DioSet(pADI_GP1,0x80);              //CS up
}

/*******************************************************************
  @brief int   ADE7913ReadOperation(unsigned char Address)
         The read operation using ADE7913 SPI interface
  @param
         unsigned char Address:              register address
         unsigned char RegisterByteNumber:   length of the register
  @return   
         unsigned int  ReceiveValue      :   reading value
*******************************************************************/
int   ADE7913ReadOperation(unsigned char Address, unsigned char RegisterByteNumber)
{
    unsigned int  SendCommand = 0;
    unsigned char SendTemp[4];
    unsigned char ReceiveTemp[4];
    unsigned int  ReceiveValue;
    unsigned char i;    
    
    SendCommand = Address<<3;
    SendTemp[0] = SendCommand|0x04; 
    for(i=0; i<RegisterByteNumber; i++)
    {
         SendTemp[i+1] = 0;
    }  
    DioClr(pADI_GP1,0x80);              //CS down
    SpiFunction(SendTemp, ReceiveTemp, (RegisterByteNumber+1)); 
    DioSet(pADI_GP1,0x80);              //CS up
    
    switch (RegisterByteNumber)
    {
        case 1:                         //if the register is one byte length
          ReceiveValue = ReceiveTemp[1];
          break;
        case 2:                         //if the register is two bytes length
          ReceiveValue = ReceiveTemp[1];
          ReceiveValue = (ReceiveValue<<8) + ReceiveTemp[2];
          break;
        case 3:                         //if the register is three bytes length
          ReceiveValue = ReceiveTemp[1];
          ReceiveValue = (ReceiveValue<<8) + ReceiveTemp[2];
          ReceiveValue = (ReceiveValue<<8) + ReceiveTemp[3];
          break;      
        default:
          break;
    }
    return(ReceiveValue);
}

/*******************************************************************
  @brief OutputRegisters ADE7913BurstRead(void)
         Reading multiple consecutive registers simultaneously in burst mode
  @param
         none
  @return   
         OutputRegisters RegisterValue: output registers value(IWV, V1WV, V2WV, ADC_CRC, STATUS0, and CNT_SNAPSHOT)
*******************************************************************/
OutputRegisters ADE7913BurstRead(void)
{
    unsigned char SendTemp[4];
    unsigned char ReceiveTemp[4];
    OutputRegisters RegisterValue;
    unsigned int  ReceiveValue;
    unsigned char i;  
    for(i=0; i<4; i++)
    {
         SendTemp[i] = 0;
    }  
    SendTemp[0] = 0x04;
    
    DioClr(pADI_GP1,0x80);              //CS down   
    
    SpiFunction(SendTemp, ReceiveTemp, 4);        //burst mode initiated and read IWV
    ReceiveValue = ReceiveTemp[1];
    ReceiveValue = (ReceiveValue<<8) + ReceiveTemp[2];
    ReceiveValue = (ReceiveValue<<8) + ReceiveTemp[3];
    RegisterValue.Current = ReceiveValue;       
    
    SendTemp[0] = 0x00;
    SpiFunction(SendTemp, ReceiveTemp, 3);        //V1WV
    ReceiveValue = ReceiveTemp[0];
    ReceiveValue = (ReceiveValue<<8) + ReceiveTemp[1];
    ReceiveValue = (ReceiveValue<<8) + ReceiveTemp[2];
    RegisterValue.Voltage1 = ReceiveValue;        
    
    SpiFunction(SendTemp, ReceiveTemp, 3);        //V2WV
    ReceiveValue = ReceiveTemp[0];
    ReceiveValue = (ReceiveValue<<8) + ReceiveTemp[1];
    ReceiveValue = (ReceiveValue<<8) + ReceiveTemp[2];
    RegisterValue.Voltage2 = ReceiveValue;    
    
    SpiFunction(SendTemp, ReceiveTemp, 2);        //ADC_CRC
    ReceiveValue = ReceiveTemp[0];
    ReceiveValue = (ReceiveValue<<8) + ReceiveTemp[1];
    RegisterValue.ADC_CRCValue = ReceiveValue;    
    
    SpiFunction(SendTemp, ReceiveTemp, 1);        //STATUS0
    RegisterValue.Status0 = ReceiveTemp[1];  
    
    SpiFunction(SendTemp, ReceiveTemp, 2);        //CNT_SNAPSHOT
    ReceiveValue = ReceiveTemp[0];
    ReceiveValue = (ReceiveValue<<8) + ReceiveTemp[1];
    RegisterValue.CounterSnapshot = ReceiveValue;    
   
    DioSet(pADI_GP1,0x80);              //CS up
    return(RegisterValue);    
}
